--	GameState.lua - Main script run by LuaGameState.cpp.
--  ----------------------------------------------------------------------------
--	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
--	apps.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

--------------------------------------------------------------------------------
--	The 4 players.
players = {}

--	Our copy of the drawing area size.
drawingSize = TwoFloats()

--	Used to clear the background to white at the start.
clearToWhite = 2
--	The colour of the background.
background = ThreeFloats(1.0)

--------------------------------------------------------------------------------
--	Called when the state is first initialised.
function initialise(drawSize)
	drawingSize = drawSize

	for i=1,4 do
		players[i] = Player(drawSize*0.5, i)
	end
end

--------------------------------------------------------------------------------
--	Used to update game logic etc.
function update(sound)
	local feedback = 1.0

	for i=1,4 do
		players[i]:update(sound)

		if players[i].erase then
			feedback = feedback - 0.2
		end
	end

	sound:setGranFeedback(feedback)
end

--------------------------------------------------------------------------------
--	Draws the game.
function draw(d)
	d:setClearBetweenFrames(false)

	if (clearToWhite > 0) then
		d:fillRect(TwoFloats(0.0), drawingSize, background, 1.0)
		clearToWhite = clearToWhite - 1
	end

	for i=1,4 do
		players[i]:draw(d)
	end
end

--------------------------------------------------------------------------------
--	Called when the user depresses a key or a joystick button.
function keyDown(action, keyPressed, keyModifier, sound)
	if (string.find(action, "P1") ~= nil) then
		players[1]:joyMove(string.sub(action, 3), 1.0, sound)
	elseif (string.find(action, "P2") ~= nil) then
		players[2]:joyMove(string.sub(action, 3), 1.0, sound)
	elseif (string.find(action, "P3") ~= nil) then
		players[3]:joyMove(string.sub(action, 3), 1.0, sound)
	elseif (string.find(action, "P4") ~= nil) then
		players[4]:joyMove(string.sub(action, 3), 1.0, sound)
	elseif (action == "Restart") then
		clearToWhite = 2
		if (background.x > 0.5) then
			background = ThreeFloats(0.0)
		else
			background = ThreeFloats(1.0)
		end

		for i=1,4 do
			players[i].eraseCol = background
		end

		sound:pickNewPitches()
	end
end

--------------------------------------------------------------------------------
--	Called when the user releases a key or a joystick button.
function keyUp(action, keyPressed, keyModifier)
	
end

--------------------------------------------------------------------------------
--	Called when the user depresses a mouse button.
function mouseDown(button, pos)
	
end

--------------------------------------------------------------------------------
--	Called when the user releases a mouse button.
function mouseUp(button, pos)
	
end

--------------------------------------------------------------------------------
--	Called when the user moves the mouse.
function mouseMove(pos)
	
end

--------------------------------------------------------------------------------
--	Called when the user moves the mousewheel.
function mouseWheel(dir)

end

--------------------------------------------------------------------------------
--	Called when the user moves a joystick.
function joyMove(action, value, joystick, axis, sound)
	if (string.find(action, "P1") ~= nil) then
		players[1]:joyMove(string.sub(action, 3), value, sound)
	elseif (string.find(action, "P2") ~= nil) then
		players[2]:joyMove(string.sub(action, 3), value, sound)
	elseif (string.find(action, "P3") ~= nil) then
		players[3]:joyMove(string.sub(action, 3), value, sound)
	elseif (string.find(action, "P4") ~= nil) then
		players[4]:joyMove(string.sub(action, 3), value, sound)
	end
end
