//	ShardsSounder.h - The game's sounder.
//  ----------------------------------------------------------------------------
//	This file is part of 'SoManyJaggedShards', an entry for 7DRL 2013.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SHARDSSOUNDER_H_
#define SHARDSSOUNDER_H_

#include "synth/BrushStrokesSynth.h"
#include "synth/PolyphonicSynth.h"
#include "synth/CombFilterBank.h"
#include "synth/SimpleSynth.h"
#include "synth/Granulator.h"
#include "sound/Sounder.h"
#include "sound/Delay.h"

#include <vector>

///	The game's sounder.
class ShardsSounder : public Sounder
{
  public:
	///	Constructor.
	ShardsSounder();
	///	Destructor.
	~ShardsSounder();

	///	Called for every audio block.
	void getAudio(float *input, float *output, int numSamples);

	///	Called for each sound file resource by LoadingScreen.
	void loadSoundFileResource(SoundFileLoaderPtr soundFileLoader) {};

	///	Resets all the synths.
	void reset();

	///	Triggers a fade out.
	void triggerFadeOut();

	///	Sets the pitch of the Brush Strokes synth.
	void setBrushStrokesPitch(int index, int val);
	///	Sets the Brush Strokes synth's table.
	void setBrushStrokesTable(int index, const TwoFloats *table);
	///	Sets the panning for the Brush Strokes synth.
	void setBrushStrokesPan(int index, float val);
	///	Sets the amplitude for the Brush Strokes synth.
	void setBrushStrokesAmp(int index, float val);

	///	Triggers the pad synth.
	void triggerPad(int val);

	///	Sets the delay time (in seconds).
	void setDelay(float val);

	///	Sets the granulator feedback.
	void setGranFeedback(float val);

	///	Sets the pitch multiplier.
	void setPitchMultiplier(float val);

	///	Increments the distortion gain.
	void incDistortGain();
	///	Returns the current distortion gain.
	float getDistortGain() const {return distortGain;};
	///	Turns the distortion on/off.
	void setLoud(bool val);
	///	Returns whether the distortion is on or off.
	bool getLoud() const {return loud;};

	///	Picks a new random pitch set (that isn't the last one).
	void pickNewPitches();
	///	Selects a new pitch set.
	void selectPitches(int index);
  private:
	///	Some constants.
	enum
	{
		BufferSize = 44100*2*2
	};

	///	The player's brush strokes synth.
	BrushStrokesSynth brushStrokes[4];
	///	The index of the current pitch for the brush strokes synth.
	int brushStrokesPitch[4];

	///	The Pow synth.
	PolyphonicSynth<SimpleSynth> padSynth;

	///	The delay on the output.
	Delay delay;
	///	The 2 granulators.
	Granulator granulators[2];
	///	The comb filter bank.
	CombFilterBank combFilters;

	///	Buffers for the 2 granulators.
	float granBuffers[2][BufferSize];

	///	All pitches are multiplied by this value.
	float pitchMultiplier;

	///	Used to distort the output.
	float distortGain;
	///	Whether to distort the output or not.
	bool loud;

	///	The available pitch sets.
	std::vector<std::vector<float> > pitchSets;
	///	The current pitch set.
	int currentPitchSet;

	///	Used to fade out when the game is quit.
	float fadeOut;
};

#endif
